#include <SoftwareSerial.h>

#include "Wire.h" // librería Wire.h
 
//Direccion I2C de la IMU
#define MPU 0x68
 
//Ratios de conversion
#define A_R 16384.0    // 32768/2
#define G_R 3640  // 32768/9
 

//MPU-6050 da los valores en enteros de 16 bits
//Valores RAW
int16_t AcX, AcY, AcZ, GyX, GyY, GyZ;

//Angulos
float Gy[3];
float Angle[3] = {90,90,90};

//Leds interrupcion
const int Verde = 5;
const int Rojo = 4;

int dato, angulo0, angulo1, angulo2;


void setup() 
{
  pinMode(Verde, OUTPUT);
  pinMode(Rojo, OUTPUT);
  attachInterrupt(digitalPinToInterrupt(2), parar, LOW);  
  Wire.begin(); // D2(GPIO4)=SDA / D1(GPIO5)=SCL
  Wire.beginTransmission(MPU);
  Wire.write(0x6B);
  Wire.write(0);
  Wire.endTransmission(true);
  Serial.begin(115200);
  digitalWrite(Verde, HIGH);
  digitalWrite(Rojo, HIGH);
  delay(500);
  digitalWrite(Verde, LOW);
  digitalWrite(Rojo, LOW);
  delay(500);
  digitalWrite(Verde, HIGH);
}


void loop() 
{

  LeerGiroscopio();

  CalcularAngulo();

  //Envia el valor del angulo 
  if (angulo0 >0 && angulo0 < 180)
  {
    
    Serial.write('f');
    delay(5);
    Serial.write(angulo0);
    delay(5);
  }

  //Envia los valores de las resistencias flexibles
  for(unsigned i=0; i<4; i++)
  {
    dato = analogRead(i);               // Lee el valor del sensor
    dato = map(dato, 0, 1023, 0, 255);    // Convierte el dato a 1 byte
    
    if (i == 0)                   // Identifica que sensor lee
    {
      Serial.write('a');         // Escrbe una letra para identificarlo    
      delay(5);                  // Espera para no saturar el buffer de memoria del bluetooth
      Serial.write(dato);        // Escribe el dato correspondiente al sensor
      delay(5);                  // Espera a que de tiempo de mover el motor por la otra placa
    }
    if (i == 1)                   // Vuelve a repetir el mismo proceso para los demas sensores
    {
      Serial.write('b');
      delay(5);
      Serial.write(dato);
      delay(5);
    }
    if (i == 2)
    {
      Serial.write('c');
      delay(5);
      Serial.write(dato);
      delay(5);
    }
    if (i == 3)
    {
      Serial.write('d');
      delay(5);
      Serial.write(dato);
      delay(5);
    }
  }
  
  dato = analogRead(6);               // Lee el valor del sensor
  dato = map(dato, 0, 1023, 0, 255);    // Convierte el dato a 1 byte
  Serial.write('e');
  delay(5);
  Serial.write(dato);
  delay(5);
}


void parar()
{
  digitalWrite(Verde, LOW);
  digitalWrite(Rojo, HIGH);
  while(digitalRead(2)==LOW)
  {
  }
  
  Angle[0] = 90;
  Angle[1] = 90;
  Angle[2] = 90;
  
  digitalWrite(Rojo, LOW);
  digitalWrite(Verde, HIGH);
}


//Lee los valores del giroscopio

void LeerGiroscopio()
{
  //Leer los valores del Giroscopio
   Wire.beginTransmission(MPU);
   Wire.write(0x43);
   Wire.endTransmission(false);
   Wire.requestFrom(MPU,6,true);   //A partir del 0x43, se piden 6 registros
   GyX=Wire.read()<<8|Wire.read(); //Cada valor ocupa 2 registros
   GyY=Wire.read()<<8|Wire.read();
   GyZ=Wire.read()<<8|Wire.read();
}

  //Calculo del angulo 
  
void CalcularAngulo()
{
    
   
  Gy[0] = GyX/G_R;
  Gy[1] = GyY/G_R;
  Gy[2] = GyZ/G_R; 

  //Sumar la variacion de angulo al angulo total.
  
  Angle[0]+=Gy[0]/3;
  Angle[1]+=Gy[1]/3;
  Angle[2]+=Gy[2]/3;

  angulo0 = Angle[0];
  angulo1 = Angle[1];
  angulo2 = Angle[2];

  
}

