#include <SoftwareSerial.h>

#include "Wire.h" // librería Wire.h
 
//Direccion I2C de la IMU
#define MPU 0x68
 
//Ratios de conversion
#define A_R 16384.0    // 32768/2
#define G_R 3640  // 32768/9
 
//Conversion de radianes a grados 180/PI
#define RAD_A_DEG = 57.295779
 
//MPU-6050 da los valores en enteros de 16 bits
//Valores RAW
int16_t AcX, AcY, AcZ, GyX, GyY, GyZ;

//Angulos
float Acc[3];
float Gy[3];
float Vel[3] = {0,0,0};
float Pos[3] = {90,90,90};
float Angle[3] = {90,90,90};


//Flex Sensor
int dato;
char datochar;
 
const int Verde = 5;
const int Rojo = 4;

int lectura, angulo0, angulo1, angulo2;
char lecturachar;

void setup()
{
  pinMode(Verde, OUTPUT);
  pinMode(Rojo, OUTPUT);
  attachInterrupt(digitalPinToInterrupt(2), parar, LOW);  //Define la interrupcion
  Wire.begin(); // D2(GPIO4)=SDA / D1(GPIO5)=SCL
  Wire.beginTransmission(MPU);
  Wire.write(0x6B);
  Wire.write(0);
  Wire.endTransmission(true);
  Serial.begin(115200);
  digitalWrite(Verde, HIGH);
  digitalWrite(Rojo, HIGH);
  delay(500);
  digitalWrite(Verde, LOW);
  digitalWrite(Rojo, LOW);
  delay(500);
  digitalWrite(Verde, HIGH);
}

void loop()
{
  
  LeerGiroscopio();

  CalcularAngulo();
      
  if (angulo0 >0 && angulo0 < 180)
  {
    
    Serial.write('a');
    delay(5);
    Serial.write(angulo0);
    delay(5);
  }
        
  if (angulo1 >0 && angulo1 < 180)
  {
    
    Serial.write('b');
    delay(5);
    Serial.write(angulo1);
    delay(5);
  }


/*  
  LeerAcelerometro();
  CalcularPosicionAcc();

  //Imprimir valores en grafica
 
  Serial.print(0);
  Serial.print(",");
  Serial.print(10);
  Serial.print("  ,  ");
  Serial.print(Acc[0]);
  Serial.print("  ,  ");
  Serial.print(Acc[1]);
  Serial.print("  ,  ");
  Serial.print(Acc[2]);
  Serial.print("  ,  ");
  Serial.println(-10);

 */
}


// Interrupcion al pulsar el boton y resetea el valor del angulo

void parar()
{
  digitalWrite(Verde, LOW);
  digitalWrite(Rojo, HIGH);
  while(digitalRead(2)==LOW)
  {
  }
  
  Angle[0] = 90;
  Angle[1] = 90;
  Angle[2] = 90;
  
  digitalWrite(Rojo, LOW);
  digitalWrite(Verde, HIGH);
}


//Lee los valores del acelerometro

void LeerAcelerometro()
{
  //Leer los valores del Acelerometro de la IMU
   Wire.beginTransmission(MPU);
   Wire.write(0x3B); //Pedir el registro 0x3B - corresponde al AcX
   Wire.endTransmission(false);
   Wire.requestFrom(MPU,6,true);   //A partir del 0x3B, se piden 6 registros
   AcX=Wire.read()<<8|Wire.read(); //Cada valor ocupa 2 registros
   AcY=Wire.read()<<8|Wire.read();
   AcZ=Wire.read()<<8|Wire.read();
}


//Lee los valores del giroscopio

void LeerGiroscopio()
{
  //Leer los valores del Giroscopio
   Wire.beginTransmission(MPU);
   Wire.write(0x43);
   Wire.endTransmission(false);
   Wire.requestFrom(MPU,6,true);   //A partir del 0x43, se piden 6 registros
   GyX=Wire.read()<<8|Wire.read(); //Cada valor ocupa 2 registros
   GyY=Wire.read()<<8|Wire.read();
   GyZ=Wire.read()<<8|Wire.read();
}

  //Calculo del angulo 
  
void CalcularAngulo()
{
    
   
  Gy[0] = GyX/G_R;
  Gy[1] = GyY/G_R;
  Gy[2] = GyZ/G_R; 

  //Sumar la variacion de angulo al angulo total.
  
  Angle[0]+=Gy[0]/3;
  Angle[1]+=Gy[1]/3;
  Angle[2]+=Gy[2]/3;

  angulo0 = Angle[0];
  angulo1 = Angle[1];
  angulo2 = Angle[2];

}


void CalcularPosicionAcc()
{

  //Transfomrar los valores a m/s2
  
  Acc[0] = AcX/A_R*9.8;
  Acc[1] = AcY/A_R*9.8;
  Acc[2] = AcZ/A_R*9.8;

  //Pasar la aceleracion a velocidad

  Vel[0] += Acc[0];
  Vel[1] += Acc[1];
  Vel[2] += Acc[2];

  //Pasar la velocidad a posicion

  Pos[0] += Vel[0];
  Pos[1] += Vel[1];
  Pos[2] += Vel[2];

}




